@@ -1,24 +1,20 @@
-// Copyright (C) 2000-2004, Luca Padovani <luca.padovani@cs.unibo.it>.
-//
-// This file is part of GtkMathView, a Gtk widget for MathML.
+// Copyright (C) 2000-2007, Luca Padovani <padovani@sti.uniurb.it>.
 // 
-// GtkMathView is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
-//
-// GtkMathView is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with GtkMathView; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// This file is part of GtkMathView, a flexible, high-quality rendering
+// engine for MathML documents.
 // 
-// For details, see the GtkMathView World-Wide-Web page,
-// http://helm.cs.unibo.it/mml-widget/, or send a mail to
-// <lpadovan@cs.unibo.it>
+// GtkMathView is free software; you can redistribute it and/or modify it
+// under the terms of the GNU Lesser General Public License as published
+// by the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
+// 
+// GtkMathView is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <config.h>
 
@@ -33,6 +29,7 @@
 #include "ShaperManager.hh"
 #include "Configuration.hh"
 
+
 enum
   {
     NIL = ComputerModernFamily::FE_NIL,
@@ -114,6 +111,7 @@ static ComputerModernShaper::PlainChar cmmMap[] =
     { 0x0079, 0x79 },  // LATIN SMALL LETTER Y
     { 0x007A, 0x7A },  // LATIN SMALL LETTER Z
     { 0x0131, 0x7B },  // LATIN SMALL LETTER DOTLESS I
+    { 0x0237, 0x7C },  // LATIN SMALL LETTER DOTLESS J
     { 0x025B, 0x22 },  // LATIN SMALL LETTER OPEN E, epsiv, varepsilon
     { 0x03B5, 0x22 },  // LATIN SMALL LETTER OPEN E, epsiv, varepsilon
     { 0x0311, 0x7F },  // COMBINING INVERTED BREVE
@@ -404,6 +402,7 @@ static ComputerModernShaper::PlainChar cmrMap[] =
     { 0x00E6, 0x1A },  // LATIN SMALL LETTER AE
     { 0x00F8, 0x1C },  // LATIN SMALL LETTER O WITH STROKE
     { 0x0131, 0x10 },  // LATIN SMALL LETTER DOTLESS I
+    { 0x0237, 0x11 },  // LATIN SMALL LETTER DOTLESS J
     { 0x0152, 0x1E },  // LATIN CAPITAL LIGATURE OE
     { 0x0153, 0x1B },  // LATIN SMALL LIGATURE OE
     { 0x0300, 0x12 },  // COMBINING GRAVE ACCENT
@@ -1207,7 +1206,24 @@ static ComputerModernShaper::HBigChar wideTildeMap =
 #define OVERBRACE  0XFE37
 #define UNDERBRACE 0XFE38
 
-static Char16 specials[] = { WIDEHAT, WIDETILDE, OVERLINE, UNDERLINE, OVERBRACE, UNDERBRACE };
+static Char16 special[] = { WIDEHAT, WIDETILDE, OVERLINE, UNDERLINE, OVERBRACE, UNDERBRACE };
+
+static ComputerModernShaper::CombiningChar combining[] =
+  {
+    { 0x0300, {CMR, 0x12} },  // COMBINING GRAVE ACCENT
+    { 0x0301, {CMR, 0x13} },  // COMBINING ACUTE ACCENT
+    { 0x0302, {CMR, 0x5E} },  // COMBINING CIRCUMFLEX ACCENT
+    { 0x0303, {CMR, 0x7E} },  // COMBINING TILDE
+    { 0x0304, {CMR, 0x16} },  // COMBINING BAR
+    { 0x0305, {CMR, 0x16} },  // COMBINING OVERLINE
+    { 0x0306, {CMR, 0x15} },  // COMBINING BREVE
+    { 0x0307, {CMR, 0x5F} },  // COMBINING DOT ABOVE
+    { 0x0308, {CMR, 0x7F} },  // COMBINING DIAERESIS
+    { 0x030A, {CMR, 0x17} },  // COMBINING RING ABOVE
+    { 0x030B, {CMR, 0x7D} },  // COMBINING DOUBLE ACUTE ACCENT
+    { 0x030C, {CMR, 0x14} },  // COMBINING CARON
+    { 0x0327, {CMR, 0x18} },  // COMBINING CEDILLA
+  };
 
 #if 0
 static UChar8 cmexTTFMap[] =
@@ -1624,8 +1640,11 @@ ComputerModernShaper::registerShaper(const SmartPtr<ShaperManager>& sm, unsigned
   for (unsigned i = 0; hMap[i].ch; i++)
     sm->registerStretchyChar(hMap[i].ch, GlyphSpec(shaperId, makeFontId(ComputerModernFamily::FE_H_STRETCHY), i));
 
-  for (unsigned i = 0; i < sizeof(specials) / sizeof(Char16); i++)
-    sm->registerStretchyChar(specials[i], GlyphSpec(shaperId, makeFontId(ComputerModernFamily::FE_SPECIAL), 0));
+  for (unsigned i = 0; i < sizeof(special) / sizeof(Char16); i++)
+    sm->registerStretchyChar(special[i], GlyphSpec(shaperId, makeFontId(ComputerModernFamily::FE_SPECIAL_STRETCHY), 0));
+
+  for (unsigned i = 0; i < sizeof(combining) / sizeof(ComputerModernShaper::CombiningChar); i++)
+    sm->registerChar(combining[i].ch, GlyphSpec(shaperId, makeFontId(ComputerModernFamily::FE_COMBINING), i));
 }
 
 void
@@ -1639,12 +1658,12 @@ ComputerModernShaper::shape(ShapingContext& context) const
 {
   for (unsigned n = context.chunkSize(); n > 0; n--)
     {
-      AreaRef res;
-      const ComputerModernFamily::FontEncId encId = encIdOfFontId(context.getSpec().getFontId());
+      bool res = false;
+      const ComputerModernFamily::FontEncId encId = encIdOfFontId(context.getSpec().getFontId()); 
       switch (encId)
 	{
-	case ComputerModernFamily::FE_SPECIAL:
-	  res = shapeSpecialChar(context);
+	case ComputerModernFamily::FE_COMBINING:
+	  res = shapeCombiningChar(context);
 	  break;
 	case ComputerModernFamily::FE_H_STRETCHY:
 	  res = shapeStretchyCharH(context);
@@ -1652,6 +1671,9 @@ ComputerModernShaper::shape(ShapingContext& context) const
 	case ComputerModernFamily::FE_V_STRETCHY:
 	  res = shapeStretchyCharV(context);
 	  break;
+	case ComputerModernFamily::FE_SPECIAL_STRETCHY:
+	  res = shapeSpecialStretchyChar(context);
+	  break;
 	default:
 	  res = shapeChar(context, encId);
 	  break;
@@ -1659,8 +1681,6 @@ ComputerModernShaper::shape(ShapingContext& context) const
 
       if (!res) break;
 
-      context.pushArea(1, res);
-
       switch (getPostShapingMode())
 	{
 	case POST_SHAPING_ALWAYS:
@@ -1684,17 +1704,18 @@ void
 ComputerModernShaper::postShape(ShapingContext&) const
 { /* do nothing */ }
 
-AreaRef
-ComputerModernShaper::shapeChar(const ShapingContext& context, ComputerModernFamily::FontEncId encId) const
+bool
+ComputerModernShaper::shapeChar(ShapingContext& context, ComputerModernFamily::FontEncId encId) const
 {
   const MathVariant hardVariant = variantOfFontId(context.getSpec().getFontId());
   const MathVariant softVariant = context.getMathVariant();
   const MathVariant variant = (hardVariant != NORMAL_VARIANT) ? hardVariant : softVariant;
-  return getGlyphArea(variant, encId, context.getSpec().getGlyphId(), context.getSize());
+  context.pushArea(1, getGlyphArea(variant, encId, context.getSpec().getGlyphId(), context.getSize()));
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeStretchyCharV(const ShapingContext& context) const
+bool
+ComputerModernShaper::shapeStretchyCharV(ShapingContext& context) const
 {
   const MathVariant variant = context.getMathVariant();
   const SmartPtr<AreaFactory> factory = context.getFactory();
@@ -1707,7 +1728,10 @@ ComputerModernShaper::shapeStretchyCharV(const ShapingContext& context) const
     if (AreaRef tryNormal = getGlyphArea(variant, charSpec.normal[i], size))
       {
 	if (tryNormal->box().verticalExtent() >= span)
-	  return tryNormal;
+	  {
+	    context.pushArea(1, tryNormal);
+	    return true;
+	  }
 	else
 	  normal = tryNormal;
       }
@@ -1717,11 +1741,13 @@ ComputerModernShaper::shapeStretchyCharV(const ShapingContext& context) const
   AreaRef middle = getGlyphArea(variant, charSpec.middle, size);
   AreaRef bottom = getGlyphArea(variant, charSpec.bottom, size);
 
-  return composeStretchyCharV(factory, normal, top, glue, middle, bottom, span);
+  context.pushArea(1, composeStretchyCharV(factory, normal, top, glue, middle, bottom, span));
+
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeStretchyCharH(const ShapingContext& context) const
+bool
+ComputerModernShaper::shapeStretchyCharH(ShapingContext& context) const
 {
   const MathVariant variant = context.getMathVariant();
   const SmartPtr<AreaFactory> factory = context.getFactory();
@@ -1734,11 +1760,13 @@ ComputerModernShaper::shapeStretchyCharH(const ShapingContext& context) const
   AreaRef glue = getGlyphArea(variant, charSpec.glue, size);
   AreaRef right = getGlyphArea(variant, charSpec.right, size);
 
-  return composeStretchyCharH(factory, normal, left, glue, right, span);
+  context.pushArea(1, composeStretchyCharH(factory, normal, left, glue, right, span));
+
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeHorizontalBar(const ShapingContext& context) const
+bool
+ComputerModernShaper::shapeHorizontalBar(ShapingContext& context) const
 {
   const MathVariant variant = context.getMathVariant();
   const SmartPtr<AreaFactory> factory = context.getFactory();
@@ -1751,11 +1779,13 @@ ComputerModernShaper::shapeHorizontalBar(const ShapingContext& context) const
   barC.reserve(2);
   barC.push_back(factory->verticalSpace(thickness, 0));
   barC.push_back(factory->horizontalSpace(span));
-  return factory->ink(factory->horizontalArray(barC));
+  context.pushArea(1, factory->ink(factory->horizontalArray(barC)));
+
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeHorizontalBrace(const ShapingContext& context) const
+bool
+ComputerModernShaper::shapeHorizontalBrace(ShapingContext& context) const
 {
   const MathVariant variant = context.getMathVariant();
   const SmartPtr<AreaFactory> factory = context.getFactory();
@@ -1806,11 +1836,13 @@ ComputerModernShaper::shapeHorizontalBrace(const ShapingContext& context) const
       assert(false);
     }
 
-  return factory->horizontalArray(c);
+  context.pushArea(1, factory->horizontalArray(c));
+
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeBigCharH(const ShapingContext& context, const ComputerModernShaper::HBigChar& map) const
+bool
+ComputerModernShaper::shapeBigCharH(ShapingContext& context, const ComputerModernShaper::HBigChar& map) const
 {
   const MathVariant variant = context.getMathVariant();
   const SmartPtr<AreaFactory> factory = context.getFactory();
@@ -1837,11 +1869,13 @@ ComputerModernShaper::shapeBigCharH(const ShapingContext& context, const Compute
   accent.push_back(factory->verticalSpace(-ex->box().height, 0));
   accent.push_back(factory->shift(accentGlyph, -ex->box().height));
 
-  return factory->verticalArray(accent, 1);
+  context.pushArea(1, factory->verticalArray(accent, 1));
+
+  return true;
 }
 
-AreaRef
-ComputerModernShaper::shapeSpecialChar(const ShapingContext& context) const
+bool
+ComputerModernShaper::shapeSpecialStretchyChar(ShapingContext& context) const
 {
   switch (context.thisChar())
     {
@@ -1856,10 +1890,59 @@ ComputerModernShaper::shapeSpecialChar(const ShapingContext& context) const
     case UNDERBRACE:
       return shapeHorizontalBrace(context);
     default:
-      assert(false);
+      assert(false); /* NOT IMPLEMENTED */
+      return false;
     }
 }
 
+#include <stdio.h>
+
+bool
+ComputerModernShaper::shapeCombiningChar(ShapingContext& context) const
+{
+  const MathVariant variant = context.getMathVariant();
+  const SmartPtr<AreaFactory> factory = context.getFactory();
+  const scaled size = context.getSize();
+  const CombiningChar& charSpec = combining[context.getSpec().getGlyphId()];
+
+  if (context.nAreas() == 0)
+    {
+      context.pushArea(1, getGlyphArea(variant, 
+				       ComputerModernFamily::FontEncId(charSpec.spec.fontEnc), 
+				       charSpec.spec.index, size));
+      return true;
+    }
+
+  CharIndex n;
+  const AreaRef base = context.popArea(n);
+  const AreaRef accentGlyph = getGlyphArea(variant, ComputerModernFamily::FontEncId(charSpec.spec.fontEnc), charSpec.spec.index, size);
+
+  AreaRef accent = NULL;
+  AreaRef under = NULL;
+  scaled dx;
+  scaled dy;
+  scaled dxUnder;
+  
+  // The code "0x18" rappresent the index of Cedilla, that is the only combining char below.
+  // This is not general condition.
+
+  if (charSpec.spec.index == 0x18)
+    {
+      under = accentGlyph;
+      computeCombiningCharOffsetsBelow(base, under, dxUnder);
+    }
+  else
+    {
+      accent = accentGlyph;
+      computeCombiningCharOffsetsAbove(base, accent, dx, dy);
+    }
+
+  context.pushArea(n + 1, factory->combinedGlyph(base, accent, under, 
+						 dx, dy, dxUnder));
+					
+  return true;
+}
+
 AreaRef
 ComputerModernShaper::getGlyphArea(MathVariant variant, const GlyphIndex& glyph, const scaled& size) const
 {
@@ -1872,7 +1955,7 @@ ComputerModernShaper::getGlyphArea(MathVariant variant, const GlyphIndex& glyph,
 AreaRef
 ComputerModernShaper::getGlyphArea(MathVariant variant, ComputerModernFamily::FontEncId encId, UChar8 index, const scaled& size) const
 {
-  scaled finalSize = size;
+  scaled finalSize = size; 
   ComputerModernFamily::FontSizeId designSize;
   const ComputerModernFamily::FontNameId fontNameId = family->findFont(variant, encId, finalSize, designSize);
   if (fontNameId != ComputerModernFamily::FN_NIL)
