@@ -1,24 +1,20 @@
-// Copyright (C) 2000-2003, Luca Padovani <luca.padovani@cs.unibo.it>.
-//
-// This file is part of GtkMathView, a Gtk widget for MathML.
+// Copyright (C) 2000-2007, Luca Padovani <padovani@sti.uniurb.it>.
 // 
-// GtkMathView is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
-//
-// GtkMathView is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with GtkMathView; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// This file is part of GtkMathView, a flexible, high-quality rendering
+// engine for MathML documents.
 // 
-// For details, see the GtkMathView World-Wide-Web page,
-// http://helm.cs.unibo.it/mml-widget, or send a mail to
-// <luca.padovani@cs.unibo.it>
+// GtkMathView is free software; you can redistribute it and/or modify it
+// under the terms of the GNU Lesser General Public License as published
+// by the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
+// 
+// GtkMathView is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <config.h>
 
@@ -30,6 +26,9 @@
 #include "MathVariantMap.hh"
 #include "FormattingContext.hh"
 #include "ShaperManager.hh"
+#include "String.hh"
+#include "Area.hh"
+#include "GlyphArea.hh"
 
 MathGraphicDevice::MathGraphicDevice(const SmartPtr<AbstractLogger>& logger)
   : GraphicDevice(logger)
@@ -449,12 +448,106 @@ MathGraphicDevice::multiScripts(const FormattingContext& context,
   return getFactory()->horizontalArray(h);
 }
 
+#if 0
+#include <iostream>
+#include <stdio.h>
+#endif
+
 AreaRef
 MathGraphicDevice::underOver(const FormattingContext& context,
 			     const AreaRef& base,
 			     const AreaRef& underScript, bool accentUnder,
 			     const AreaRef& overScript, bool accent) const
 {
+  SmartPtr<const GlyphStringArea> baseStringArea = base ? base->getGlyphStringArea() : NULL;
+  SmartPtr<const GlyphStringArea> overStringArea = overScript ? overScript->getGlyphStringArea()
+						 	        : NULL;
+  SmartPtr<const GlyphStringArea> underStringArea = underScript ? 
+						    underScript->getGlyphStringArea() : NULL;
+
+#if 0
+  printf("base GlyphArea %s\n", base->getGlyphArea() ? "true" : "false");
+  printf("over GlyphArea %s\n", overScript->getGlyphArea() ? "true" : "false");  
+
+  printf("base GlyphStringArea %s\n", base->getGlyphStringArea() ? "true" : "false"); 
+  printf("over GlyphStringArea %s\n", base->getGlyphStringArea() ? "true" : "false"); 
+#endif
+
+  if (baseStringArea)
+  {
+    UCS4String baseSource;
+    UCS4String overSource;
+    UCS4String underSource;
+    AreaRef res;
+    
+    //controls if overScript and/or underScript are very sinlge character
+    bool overCondition = overStringArea && accent && 
+			 ((overSource = overStringArea->getSource()).length() == 1);
+    bool underCondition = underStringArea && accentUnder &&
+	   		  ((underSource = underStringArea->getSource()).length() == 1);
+
+    baseSource = baseStringArea->getSource();
+    //controls if the base character is a single char
+    if (baseSource.length() == 1)
+    {
+      if (overScript && underScript)
+      {
+        if (overCondition)
+        {
+	  //we have that overScript is a single character
+	  res = shaperManager->compose(context,
+				       base, baseSource,
+				       overScript, overSource, true);
+
+	  //if this condition is true then also underScript is a single char
+	  if (underCondition)
+            res = shaperManager->compose(context,
+					 res, baseSource,
+					 underScript, underSource, false);
+
+          //we have that the overScript is a single char
+          //but the underScript isn't a single char
+          else
+	    res = underOver(context, res, underScript, accentUnder, NULL, accent);  
+
+	  return res;
+        }
+        //we have that only the underScript is a single char  
+        else if (underCondition)
+	{
+	  res = shaperManager->compose(context,
+				       base, baseSource,
+				       underScript, underSource, false);
+	  res = underOver(context, res, NULL, accentUnder, overScript, accent);   
+
+	  return res;
+        }
+      }
+
+      //we control if one of overScript and underScript
+      //is a single char
+      else if (overScript || underScript)
+      {
+	if (overCondition)
+	{
+          res = shaperManager->compose(context,
+		     	  	       base, baseSource,
+			               overScript, overSource, true);
+          return res;
+        }
+        if (underCondition)
+	{
+	  res = shaperManager->compose(context,
+		     	  	       base, baseSource,
+			               underScript, underSource, false);
+          return res;
+        }
+      }
+    }
+  }
+  
+  //the next instructions represent the default behavior
+  //in which overScript and underScript aren't single char
   const scaled RULE = defaultLineThickness(context);
   const AreaRef singleSpace = getFactory()->verticalSpace(RULE, 0);
   const AreaRef tripleSpace = getFactory()->verticalSpace(3 * RULE, 0);
@@ -466,18 +559,18 @@ MathGraphicDevice::underOver(const FormattingContext& context,
   std::vector<AreaRef> v;
   v.reserve(n);
   if (underScript)
-    {
-      if (!accentUnder) v.push_back(singleSpace);
-      v.push_back(getFactory()->center(underScript));
-      if (!accentUnder) v.push_back(tripleSpace);
-    }
+  {
+    if (!accentUnder) v.push_back(singleSpace);
+    v.push_back(getFactory()->center(underScript));
+    if (!accentUnder) v.push_back(tripleSpace);
+  }
   v.push_back(getFactory()->center(base));
   if (overScript)
-    {
-      if (!accent) v.push_back(tripleSpace);
-      v.push_back(getFactory()->center(overScript));
-      if (!accent) v.push_back(singleSpace);
-    }
+  {
+    if (!accent) v.push_back(tripleSpace);
+    v.push_back(getFactory()->center(overScript));
+    if (!accent) v.push_back(singleSpace);
+  }
 
   return getFactory()->verticalArray(v, underScript ? (accentUnder ? 1 : 3) : 0);
 }
