@@ -1,24 +1,20 @@
-// Copyright (C) 2000-2005, Luca Padovani <luca.padovani@cs.unibo.it>.
-//
-// This file is part of GtkMathView, a Gtk widget for MathML.
+// Copyright (C) 2000-2007, Luca Padovani <padovani@sti.uniurb.it>.
 // 
-// GtkMathView is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License
-// as published by the Free Software Foundation; either version 2
-// of the License, or (at your option) any later version.
-//
-// GtkMathView is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with GtkMathView; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// This file is part of GtkMathView, a flexible, high-quality rendering
+// engine for MathML documents.
 // 
-// For details, see the GtkMathView World-Wide-Web page,
-// http://helm.cs.unibo.it/mml-widget/, or send a mail to
-// <lpadovan@cs.unibo.it>
+// GtkMathView is free software; you can redistribute it and/or modify it
+// under the terms of the GNU Lesser General Public License as published
+// by the Free Software Foundation; either version 3 of the License, or
+// (at your option) any later version.
+// 
+// GtkMathView is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// Lesser General Public License for more details.
+// 
+// You should have received a copy of the GNU Lesser General Public License
+// along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <config.h>
 
@@ -53,7 +49,7 @@ void
 NullShaper::shape(ShapingContext& context) const
 {
   assert(!context.done());
-  logger->out(LOG_WARNING, "shaping U+%04X as null", context.thisChar());
+  logger->out(LOG_WARNING, "shaping U+%04X as a red box", context.thisChar());
   SmartPtr<AreaFactory> factory = context.getFactory();
   std::vector<AreaRef> c;
   c.reserve(2);
